@@ -1,6 +1,6 @@
 package Apache::MP3::Resample;
 
-# $Id: Resample.pm,v 1.4 2002/08/18 17:33:49 lstein Exp $
+# $Id: Resample.pm,v 1.5 2002/08/26 18:52:29 lstein Exp $
 # Resamples (downsamples) on the fly
 
 use strict;
@@ -10,11 +10,15 @@ use IO::File;
 use CGI qw(:standard *table *TR *td escape);
 use CGI::Cookie;
 use Apache::MP3::Playlist;
+use File::Basename;
 
 @ISA = 'Apache::MP3::Playlist';
-$VERSION = '1.10';
 
-use constant ENCODE => qq(lame --mp3input %b - <%f);
+use constant ENCODE => qq(%Dlame%I %b -%F);
+
+my @DECOMPRESSOR_DEFAULTS = ("audio/shorten" => "shorten -x %f -|",
+			     "audio/flac" => "flac -d -c -s %f|");
+
 my @PRESET_DEFAULTS = (
 	       '24 kbps' => '-b24 --lowpass 4.0  --highpass 0.1  --resample 8',
 	       '40 kbps' => '-b40 --lowpass 4.0  --resample 16',
@@ -25,7 +29,22 @@ my @PRESET_DEFAULTS = (
 	       '160 kbps' => '-b160',
 	       '192 kbps' => '-b192',
 	      );
-my (%PRESETS,@PRESETS);
+
+my (%PRESETS,@PRESETS,%DECOMPRESSORS,@DECOMPRESSORS);
+
+sub new {
+   my $proto = shift;
+   my $class = ref ($proto) || $proto;
+   my $self = $class->SUPER::new (@_);
+
+   # Add to suffix list and supported type hash
+   push (@{$self->{'suffixes'}}, ".shn", ".SHN", ".flac", ".FLAC");
+   $self->{'supported_types'}->{'audio/shorten'} = 'read_shorten';
+   $self->{'supported_types'}->{'audio/flac'} = 'read_flac';
+
+   bless ($self, $class);
+   $self;
+}
 
 sub run {
   my $self = shift;
@@ -86,6 +105,23 @@ sub presets {
   return $PRESETS{$bitrate};
 }
 
+sub decompressor {
+  my $self = shift;
+  unless (%DECOMPRESSORS) {
+    my @p;
+    if (my $conf = $self->r->dir_config('DecompressorPresets')) {
+      @p = split /\s*(?:=>|,)\s*/,$conf;
+    } else {
+      @p = @DECOMPRESSOR_DEFAULTS;
+    }
+    my @i = map {$_*2} (0..(@p-1)/2);
+    %DECOMPRESSORS    = @p;
+    @DECOMPRESSORS = @p[@i];
+  }
+  my $mime = shift or return @DECOMPRESSORS;
+  return $DECOMPRESSORS{$mime};
+}
+
 sub sample_popup {
   my $self = shift;
   my @bitrates = $self->presets;
@@ -124,52 +160,151 @@ sub open_file {
   my $file = shift;
   my $bitrate = $self->bitrate;
   my $presets = $self->presets($bitrate);
-  my $encode = $self->r->dir_config('MP3Encoder') || ENCODE;
+  my $type = $self->r->lookup_file($file)->content_type;
+  my $decompress = $self->decompressor ($type) || "";
+  my $encode = ENCODE;
+  my $inputtype = length $decompress ? "" : " --mp3input";
+  my $percentF = length $decompress ? "" : ("<" . quotemeta ($file));
+  $decompress =~ s{%([a-zA-Z])}
+    { $1 eq 'f' ? quotemeta($file) :
+	"%$1"}exg;
   $encode =~ s{%([a-zA-Z])}
               {$1 eq 'b' ? $presets         :
+	       $1 eq 'D' ? $decompress      :
+	       $1 eq 'I' ? $inputtype       :
+	       $1 eq 'F' ? $percentF        :
                $1 eq 'f' ? quotemeta($file) :
                "%$1"}exg;
-  my $filter = $self->r->dir_config('VerboseMP3Encoder') 
+  my $filter = $self->r->dir_config('VerboseMP3Encoder')
     ? "$encode |" : "$encode 2>/dev/null |";
   return IO::File->new($filter);
 }
 
+sub read_shorten {
+  my $self = shift;
+  my ($file,$data) = @_;
+  my $sec;
+  my $bits = 0;
+  my $samples = 0;
+  my $channels = 0;
+  my $track = "";
+
+  open (SHNINFO, "shntool info \Q$file\E 2>/dev/null|") or return;
+
+  while (<SHNINFO>) {
+     if (/^length:\s+(\d+):(\d+)\.(\d+)/) {
+	$sec = $1 * 60 + $2;
+     } elsif (/^channels:\s+(\d+)/) {
+	$channels = $1;
+     } elsif (/^samples\/sec:\s+(\d+)/) {
+	$samples = $1;
+     } elsif (/^bits\/sample:\s+(\d+)/) {
+	$bits = $1;
+     }
+  }
+  close SHNINFO;
+
+  %$data = (
+	    title  => basename ($file),
+	    artist => "",
+	    album  => "",
+	    year   => "",
+	    genre  => "",
+	    track  => $track,
+	    comment => "",
+	    min         => int $sec/60,
+	    sec         => $sec % 60,
+	    seconds     => $sec,
+	    bitrate     => int ($channels * $bits * $samples / 1024),
+	    samplerate  => $samples,
+	    duration    => sprintf("%dm %2.2ds", int $sec/60,$sec%60),
+	   )
+}
+
+sub read_flac {
+  my $self = shift;
+  my ($file,$data) = @_;
+  my $sec;
+  my $bits = 0;
+  my $samples = 0;
+  my $channels = 0;
+  my $track = "";
+  my $rate = 0;
+
+  open (METAFLAC, "metaflac --list --block-number 0 \Q$file\E 2>/dev/null|")
+    or return;
+
+  while (<METAFLAC>) {
+     if (/sample_rate: (\d+)\s/) {
+	$rate = $1;
+     } elsif (/channels: (\d+)/) {
+	$channels = $1;
+     } elsif (/bits-per-sample: (\d+)/) {
+	$bits = $1;
+     } elsif (/total samples: (\d+)/) {
+	$samples = $1;
+	$sec = $samples / $rate if $rate;
+     }
+  }
+  close METAFLAC;
+
+  %$data = (
+	    title  => basename ($file),
+	    artist => "",
+	    album  => "",
+	    year   => "",
+	    genre  => "",
+	    track  => $track,
+	    comment => "",
+	    min         => int $sec/60,
+	    sec         => $sec % 60,
+	    seconds     => $sec,
+	    bitrate     => int ($channels * $bits * $rate / 1024),
+	    samplerate  => $samples,
+	    duration    => sprintf("%dm %2.2ds", int $sec/60,$sec%60),
+	   )
+}
+
 1;
 
 =head1 NAME
 
-Apache::MP3::Resample - Downsample MP3 files during streaming
+Apache::MP3::Resample - Downsample MP3/FLAC/Shorten files during streaming
 
 =head1 SYNOPSIS
 
  # httpd.conf or access.conf
+ AddType audio/shorten .shn .SHN
+ AddType audio/flac .flac
+
+ Alias /apache_mp3 /usr/share/libapache-mp3-perl
+
  <Location /songs>
    SetHandler perl-script
    PerlHandler Apache::MP3::Resample
-   PerlSetVar  MP3Encoder 'lame --mp3input %b - <%f'
-   PerlSetVar  ResamplePresets 'Low     => -b16,\
-                                Medium  => -b56,\
-                                High    => -b128,\
-				Hi Fi   => -b160'
+   PerlSetVar CacheDir		/var/cache/Apache::MP3
+   PerlSetVar AllowDownload	no
+   PerlSetVar SortFields	Album,Track,Title,-Duration
+   PerlSetVar Fields		Track,Title,Artist,Album,Duration,Bitrate
  </Location>
 
 =head1 DESCRIPTION
 
 Apache::MP3::Resample subclasses Apache::MP3::Playlist to allow the
-user to downsample MP3 files before streaming them.  This allows users
+user to downsample audio files before streaming them.  This allows users
 on slower connections to stream songs.  When this module is installed,
-a menu of bitrates is presented in the upper left-hand corner of the
+a menu of bitrates is presented in the upper right-hand corner of the
 screen.  The user can choose from one of the bitrates, or select a
 mode that performs no resampling.  The selected bitrate is maintained
 in a persistent cookie so that resampling is performed whenever the
 user returns to the site.
 
 This module requires a command-line MP3 encoder to resample and
-reencode the MP3 sound data.  If not otherwise specified,
-Apache::MP3::Resample will try to use the Open Source Lame MP3
-encoder.  This utility is available at http://www.sulaco.org/mp3.
-Version 3.87 was used during the development of this module.  Your
-results with other versions may vary.
+reencode the audio data.  If not otherwise specified,
+Apache::MP3::Resample will try to use the Open Source Lame MP3 encoder.
+This utility is available at http://www.sulaco.org/mp3.  Version 3.90
+was used during the development of this module.  Your results with
+other versions may vary.
 
 When you install Lame (or the encoder of your choice), be sure to
 place it in a directory located in Apache's PATH so that the module
@@ -201,6 +336,22 @@ If not present, the following default is used:
 
   lame --mp3input %b - <%f
 
+=item DecompressorPresets
+
+A list of MIME types and decompression programs to use for each.  The
+format uses a variant of the standard Perl hash format, in which the
+keys are the MIME types for the audio files and the values are the
+command which will be used to decompress files of that type.  The
+token "%f" will be replaced by the name of the compressed audio file
+and the command should end with a pipe (|) character.  Here is a
+simple example, which is also the default list:
+
+    PerlSetVar  DecompressorPresets  '"audio/shorten" => "shorten -x %f -|",\
+                                      "audio/flac" => "flac -d -c -s %f|"'
+
+Note the use of quotation marks and backslashes to protect whitespace
+and newlines respectively.
+
 =item ResamplePresets
 
 A list of bitrates and the command-line options to pass to the
@@ -283,7 +434,7 @@ configuration variable, or the default names.  Called with an argument
 equal to the name of a preset, returns the command-line arguments to
 pass to the encoder.
 
-=item sample_opup()
+=item sample_popup()
 
 Draws the popup menu with the sample rate options.
 
@@ -299,11 +450,17 @@ as "file not found".  If you are having trouble with the downsampling,
 set the configuration variable VerboseMP3Encoder to a true value in
 order to see the standard error messages.
 
+Also, the module does not function properly unless CacheDir is set and
+points to a directory which exists and is writeable by the Apache
+server.
+
 =head1 ACKNOWLEDGEMENTS
 
 Many people have requested this feature and/or proposed implementations.
 Thank you all for your help.
 
+FLAC and Shorten support added by Caleb Epstein <cae@bklyn.org>.
+
 =head1 AUTHOR
 
 Copyright 2000, Lincoln Stein <lstein@cshl.org>.
